% commentaires de ligne avec '%'. Tout ce qui suit sur la ligne est ignoré par le parseur

% définir une relation
% '@let' ID ' = @relation [' ATTRIBUTE_LIST ']' '{' TUPLE_LIST '}'
@let PROFS1 = @relation [Nom, Prenom] {
  <Bruyère, Véronique>
  <Wijsen, Jozef>
  <Mens, Tom>
}

% les espaces sont optionnelles entre les éléments
@let PROFS2 = @relation [Nom,Prenom]{<Bruyère,Véronique><Wijsen,Jozef><Mens,Tom>}

% autre exemple de convention d'écriture pour la définition de relation
@let PROFS3 = @relation [   Nom,        Prenom    ] { 
                < Bruyère,    Véronique >
                < Wijsen,     Jozef     >
                < Mens,       Tom       > 
              }
              
% les valeurs d'un tuple sont séparées par des virgules
@let ECRIVAIN = @relation [Nom, Prenom] {
  < de la Fontaine, Jean >
  < de Musset, Alfred >
  < Higgins Clark, Mary >
  < Doyle, Arthur Conan >
}

@print PROFS1, PROFS2, PROFS3, ECRIVAIN


% les valeurs d'un tuple sont "trimmées", ce qui offre de la souplesse pour les conventions d'écriture
% si des espaces sont significatives au début ou à la fin d'une valeur, la valeur doit être entourée de guillemets doubles ou simples (apostrophes)
@let R1 = @relation [Titre, Année] {
  < "   espaces avant",               1995 >
  < "espaces après      ",            2000 >
  < "    espaces avant et après   ",  2000 >
}

@let R2 = @relation [Titre, Année] {
  < '   espaces avant'                  , 1995 >
  < 'espaces après      '               , 2000 >
  < '    espaces avant et après   '     , 2000 >
}

@print R1, R2


% les valeurs d'un tuple sont délimitées par une virgule ou le signe '>'
% si une valeur contient des virgules ou des signes '>', on peut soit :
%   - entourer la valeur de guillemets (simples ou doubles)
%   - échapper le caractère avec un backslash
@let CITATION = @relation [Citation, Auteur] {
  < "Les cons ça ose tout, c'est même à ça qu'on les reconnaît", Audiard >
  < "Je suis contre les femmes, tout contre", Sacha Guitry >
  < Les français ont du vin\, les Anglais de l'humour, Roland Topor >
}

@print CITATION

@let EXPRESSION_MATHEMATIQUE = @relation [expression] {
  < x1 < y1 >
  < x2 \> y2 >
  < "x3 > y3" >
  < "forall x1, x2, x3" >
}

@print EXPRESSION_MATHEMATIQUE

% échappement dans les valeurs d'un tuple :
%  \"   :     "
%  \'   :     '
%  \,   :     ,
%  \>   :     >
%  \\   :     \
% un backslash suivi d'un autre caractère que ceux qui précèdent n'est pas interprété
@let ECHAPPEMENT = @relation [exemple] {
  < c:\java >
  < c:\\windows >
  < Lundi\, j'ai répondu : "non\, non !" > 
  < "Mardi, j'ai répondu : \"non, non !\"" >
  < 'Jeudi, j\'ai répondu : "non, non !"' >
  < "une chaîne qui se termine par un backslash \\" >
  < une autre chaîne qui se termine par un backslash \ >
}

@print ECHAPPEMENT

% une relation peut ne contenir aucun tuple
@let VIDE1 = @relation [nom, prénom] {}

% pour une valeur d'attribut vide, il est nécessaire d'utiliser des guillemets (simple ou double)
@let EMPTY_VALUES = @relation [Nom, Prénom, Téléphone] {
  <Wijsen, Jozef, "" >
  <Bruyère, Véronique, '' >
}

@print EMPTY_VALUES

% une relation peut ne pas avoir d'attribut et être vide
@let VIDE2 = @relation [] {}

@print VIDE1, VIDE2

% une relation peut ne pas avoir d'attribut et contenir un seul tuple, le tuple vide
@let SANS_ATTR = @relation [] { < > }

% une relation avec un attribut et un seul tuple, dont la valeur d'attribut est la chaîne vide
@let AVEC_ATTR = @relation [a1] { < "" > }

@print SANS_ATTR, AVEC_ATTR


% NOTE : on fait l'hypothèse que tous les attributs appartiennent à un seul domaine, par exemple le domaine des chaînes de caractère